/* This sample code for getting weather data from OpenWeatherMap.com
* by Arduino UNO and TAIYO YUDEN's WLAN Mmodule "".
* You need to modify the variables as following:
* 1. Connection parameters for wireless access point (SSID...).
* 2. City ID and your APPID of OpenWeatherMap.com.
* 3. Timezone offset
* Refer to Aplication note in detail.
* TAIYO YUDEN CO., LTD, copyright 2016
*/

#include <SoftwareSerial.h> // Include the Software Serial library.
#include <FlexiTimer2.h> // Include timer library.
#include <Time.h> // Include time library.

// Define using SW Serial pins.
SoftwareSerial mySerial(10, 11); //RX, TX

// Define parameters
  // Wireless access point
    String ssId = "<< put your SSID here >>";
    String pass = "<< put your password here >>";
    // Securty: WPA2, IP: DHCP
  // OpenWeatherMap
    String cityId = "<< put city id here >>";
    String appId = "<< put your appid here >>";
    int tz = 9; // Timezone offset

// Processing of special characters (deg C)
char deg[3] = {0xdf, 0x43, 0x00};

// Flag of WLAN connection, 0: none, 1: connecting, 2: connected
int wcon = 0;

// Flag of data, 0: no data, 1: Waiting for data, 2: Received data
int wdata = 0;

// Define of the string buffer.
String res = "";
String data = "";

void setup() {
  // Setting of HW Serial (To display message from Arduino to PC)
  Serial.begin(9600);
  // Setting of SW Serial (Connect to WLAN module)
  mySerial.begin(9600);
  
  // Input connection parameter to access point.
  mySerial.print(F("WSTI101")); //SSID
  mySerial.println(ssId);
  mySerial.println(F("WSTI1024")); //security type: 4 = WPA2
  mySerial.print(F("WSTI103")); //password
  mySerial.println(pass);
  mySerial.println(F("WSTI1041")); //IP Addr: 1 = DHCP

  // Timer set to 20,000ms interval and start timer.
  FlexiTimer2::set(20000, flash);
  FlexiTimer2::start();

  // Initial screen display.
  Serial.println(F("Wait a moment."));
  Serial.println(F("Now connecting.."));
  Serial.println(F(""));
}

void loop() {
  // Processing of message from WLAN module
  if(mySerial.available() > 0){
    char inChar = mySerial.read();
    if (inChar == '\n'){
      res += '\n';
      // Receive weather data, and copy it to buffer
      if (res.indexOf("RCT01{") != -1){
        res.replace("RCT01{", "{");
        res.trim();
        data = res;
        wdata = 2;
        res = "";
      // HTTP status code error
      } else if (res.indexOf("NAK31") != -1){
        wcon = 2;
        data = "";
        wdata = 0;
      // HTTP connection error
      } else if (res.indexOf("NAK30") != -1){
        wcon = 2;
      // Already connected
      } else if (res.indexOf("NAK15") != -1){
        wcon = 2;
      // Success to connect
      } else if (res.indexOf("CON") != -1){
        wcon = 2;
      // Failed
      } else if (res.indexOf("NAK") != -1){
        wcon = 0;
      // Fatal error
      } else if (res.indexOf("FTL") != -1) {
        wcon =0;
        // WLAN module soft reset
        mySerial.println(F("WRST"));
      }
      //Serial.println(res); // For debug
      res = "";
    } else {
      res += inChar;
    }
  }
  // Try to connect if not connected
  if (wcon == 0){
    mySerial.println(F("WICO1"));
    wcon = 1;
  }
  // If data is available, formatting, display it.
  if (wcon ==2 && wdata == 2){
    //cut from json data
    int cnt1 = data.indexOf("description\":") + 14;
    int cnt2 = data.indexOf("\",\"icon");
    String wtr = data.substring(cnt1, cnt2);
    int cnt3 = data.indexOf("temp\":") + 6;
    float temp = (data.substring(cnt3, cnt3 + 6)).toFloat();
    int cnt4 = data.indexOf("pressure\":") + 10;
    int cnt5 = data.indexOf("humidity\":") + 10;
    int cnt6 = data.indexOf("temp_min\":") + 10;
    int pres = (data.substring(cnt4, cnt5)).toInt();
    int humi = (data.substring(cnt5, cnt6)).toInt();
    int cnt7 = data.indexOf("dt\":") + 4;
    long ltime = (data.substring(cnt7, cnt7 + 10).toInt());
    // Set of Datetime, and offset the timezone.
    setTime(ltime + (tz * 60 * 60));
    int cnt8 = data.indexOf("name\":") + 7;
    int cnt9 = data.indexOf("\",\"cod");
    String city = data.substring(cnt8, cnt9);
  
    // Output weather information to console screen
    if (year() != 1970){
      Serial.println(city);
      Serial.print(year());
      Serial.print(F("/"));
      Serial.print(month());
      Serial.print(F("/"));
      Serial.print(day());
      Serial.print(F(" "));
      if(hour() < 10){
        Serial.print(F("0"));
        Serial.print(hour());
      } else {
        Serial.print(hour());
      }
      Serial.print(F(":"));
      if(minute() < 10){
        Serial.print(F("0"));
        Serial.println(minute());
      } else {
        Serial.println(minute());
      }
      Serial.print(F("Weather: "));
      Serial.println(wtr);
      Serial.print(F("Current: "));
      Serial.print(temp);
      Serial.println(deg);
      Serial.print(F("Pressure: "));
      Serial.print(pres);
      Serial.println(F("hPa"));
      Serial.print(F("Humidity: "));
      Serial.print(humi);
      Serial.println(F("%"));
      Serial.println(F(""));

      // Return the flag.
      wdata = 0;
  
      // Delete data.
      data = "";
    }
  }
}

// Send request with timer
void flash(){
  //request if data is blank
  if (wcon == 2 && wdata == 0){
    // Send command for getting data
    mySerial.print(F("WHTG01http://api.openweathermap.org/data/2.5/weather?id="));
    mySerial.print(cityId);
    mySerial.print(F("&units=metric&appid="));
    mySerial.println(appId);
    wdata = 1;
  }
}
